#include "mainWindow.h"

#include <QApplication>
#include <Qwidget>
#include <QMainWindow>
#include <QGraphicsView>
#include <QGraphicsScene>
#include <QMenuBar>
#include <QAction>
#include <QFileDialog>
#include <QString>
#include <QDockWidget>
#include <QStandardItemModel>
#include <QStandardItem>
#include <QListView>
#include <QModelIndex>
#include <QImage>

#include <brlcad/ConstDatabase.h>
#include <brlcad/cicommon.h>

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent)
{
    scene = new QGraphicsScene(this);
    view = new QGraphicsView(scene, this);

    setCentralWidget(view);

    fileMenu = menuBar()->addMenu(tr("&File"));
    QAction *openAct = new QAction(tr("&Open..."), this);
    openAct->setStatusTip(tr("Open an existing file"));
    connect(openAct, &QAction::triggered, this, &MainWindow::openDialog);
    fileMenu->addAction(openAct);

    viewMenu = menuBar()->addMenu(tr("&View"));
    QAction *showSelectionAct = new QAction(tr("&Show Object Selection"), this);
    connect(showSelectionAct, &QAction::triggered, this, &MainWindow::createListDockWidget);
    viewMenu->addAction(showSelectionAct);
}

void MainWindow::createListDockWidget() {
    model = new QStandardItemModel(0);
    QList<QStandardItem *> list;

    BRLCAD::ConstDatabase::TopObjectIterator i = database.FirstTopObject();
    for (; i.Good(); i.operator++()) {
        QStandardItem *item = new QStandardItem(i.Name());
        item->setEditable(false);
        list << item;
    }
    model->appendColumn(list);

    QListView *dockWidgetContents = new QListView;
    dockWidgetContents->setModel(model);
    connect(dockWidgetContents, &QAbstractItemView::clicked,this, &MainWindow::objectClicked);

    if (dockWidget != 0)
        dockWidget->close();
    dockWidget = new QDockWidget(tr("Object Selection"), this);
    dockWidget->setAllowedAreas(Qt::LeftDockWidgetArea |
                                Qt::RightDockWidgetArea);
    dockWidget->setWidget(dockWidgetContents);
    addDockWidget(Qt::LeftDockWidgetArea, dockWidget);
}

void MainWindow::openDialog() {
    QString fileName = QFileDialog::getOpenFileName(this,
        tr("Open Database"), "./", tr("Database Files (*.g)"));
    QByteArray ba = fileName.toLatin1();
    char *filePath = ba.data();
    open(filePath);
}

void MainWindow::open(char *filePath) {
    if (database.Load(filePath)) {
        databaseTitle = database.Title();
    } else {
        databaseTitle = "database not found";
    }
    char title[100];
    strcpy(title, "Alternative Gui - ");
    strcat(title, databaseTitle);
    setWindowTitle(
        QApplication::translate("MainWindow", title));

    createListDockWidget();
    scene->clear();
}

void MainWindow::objectClicked(const QModelIndex &index) {
    QStandardItem *item = model->itemFromIndex(index);
    QByteArray ba = item->text().toLatin1();
    char *name = ba.data();

    database.UnSelectAll();
    database.Select(name);
    BRLCAD::Vector3D min = database.BoundingBoxMinima();
    BRLCAD::Vector3D max = database.BoundingBoxMaxima();
    QString minX("Bounding Box Min: x: ");
    QString minXInt = QString::number(min.coordinates[0]);
    QString minY(" y: ");
    QString minYInt = QString::number(min.coordinates[1]);
    QString minZ(" z: ");
    QString minZInt = QString::number(min.coordinates[2]);
    QString maxX("\nBounding Box Max: x: ");
    QString maxXInt = QString::number(max.coordinates[0]);
    QString maxY(" y: ");
    QString maxYInt = QString::number(max.coordinates[1]);
    QString maxZ(" z: ");
    QString maxZInt = QString::number(max.coordinates[2]);
    
    QString text = minX + minXInt + minY + minYInt + minZ + minZInt +
        maxX + maxXInt + maxY + maxYInt + maxZ + maxZInt;
    scene->clear();
    scene->addText(text);
}

BRLCAD::Vector2D MainWindow::bLRCordsToDACords(BRLCAD::Vector3D input) {
    BRLCAD::Vector2D renderSize(renderSize.coordinates[0] - 2 * margin, renderSize.coordinates[1] - 2 * margin);
    BRLCAD::Vector2D objectSize(max.coordinates[0] - min.coordinates[0], max.coordinates[1] - min.coordinates[1]);
    BRLCAD::Vector2D scaleSize(renderSize.coordinates[0]/objectSize.coordinates[0], renderSize.coordinates[1]/objectSize.coordinates[1]);
    
    BRLCAD::Vector2D output(scaleSize.coordinates[0] * input.coordinates[0], scaleSize.coordinates[1] * -input.coordinates[1]);

    return output;
}

BRLCAD::Vector3D MainWindow::dACordsToBLRCords(BRLCAD::Vector2D input) {
    BRLCAD::Vector2D renderSize(renderSize.coordinates[0] - 2 * margin, renderSize.coordinates[1] - 2 * margin);
    BRLCAD::Vector2D objectSize(max.coordinates[0] - min.coordinates[0], max.coordinates[1] - min.coordinates[1]);
    BRLCAD::Vector2D scaleSize(objectSize.coordinates[0]/renderSize.coordinates[0], objectSize.coordinates[1]/renderSize.coordinates[1]);

    BRLCAD::Vector3D output(scaleSize.coordinates[0] * input.coordinates[0] + min.coordinates[0],
        scaleSize.coordinates[1] * -input.coordinates[1] + min.coordinates[1], min.coordinates[2] + 5);

    return output;
}

void MainWindow::drawSelectedObjectRayTrace() {
    QImage rayTrace(renderSize.coordinates[0], renderSize.coordinates[1]);
    for (int i=margin; i<renderSize.coordinates[1]-margin; i++) {
        for (int j=margin; j<renderSize.coordinates[0]-margin; j++) {
           
        }
    }
}